/*
 * fcomp.h
 * Fox Compression header
 * GPL license
 */
/*
Here are the specs for both formats.

fcomp-standard:
TYPE   SIZE    DESC
byte      1    Pointer to the next backpointer.  0 is EOS
stream    X    a bunch of bytes to take as pure stream
byte      1    Distance.  It is 1-256 bytes back look (not 0-255)
byte      1    Length.  Size to copy, 0-255.  0 allows for chaining
               backpointers in the stream when there is a lot of nonredundant
	       data

This repeats until EOS (which is likely a 0 0 0 if the end of the stream was
not found in the previous 256 bytes; if it was, then it's just D L 0).

fcomp-extended:
TYPE   SIZE    DESC
byte      1    0.  This indicates EOS in an fcomp-standard, thus since this
               makes absolutely no sense it signifies an fcomp-extended.
byte      1    Tells how many bytes the pointers are.  One only: beginning of
               the stream.  This is N
N         N    Tell what the window is.  This is the max distance for a
               backpointer.
N         N    Distance forward to next backpointer.  N bytes in size. 0 is EOS
stream    X    A bunch of bytes to take as pure data stream
N         N    Distance back to go to get data.  Base 1
N         N    Length of data to copy.  Base 0

The first three are the header (2+N bytes); the rest repeat until EOS.
*/

#ifndef _FCOMP__H_
#define _FCOMP__H_

// FComp
#define FC_LOCK 0x01
#define FC_BPC  0x02
#define FC_EOS  0x80

// FComp Function
#define FCF_ERROR 0x80000000

struct fcomp
{
  int8_t       flags; // Flags
                     // 1 - Lock
		     // 2 - BackPointer Check (ended at barrier)
		     // 128 - End Of Stream
  int8_t       psize; // Pointer size.  1 is automatically a fcomp-standard
  int32_t      mdist; // max distance
  int32_t      rdist; // real distance, for when abuf is filling.
  int32_t      lob; //  Length of buffer stream.
  char*        stream; // Stream
  char*        abuf; // analysis buffer, mdist+1 bytes
  int32_t      last_pointer; // Position in the stream of the last pointer
};

struct fdecomp
{
  int8_t       flags; // Flags.  1 - Lock.
  int8_t       psize; // pointer size.  1 unless fcomp-extended.
  int32_t      mdist; // max distance back
  int32_t      rdist; // Real length of the stream
  int32_t      lob;   // Length of buffer
  char*        stream; // stream
  char*        abuf;  // analysis buffer.
  int32_t      lop;   // length of pbuf
  char*        pbuf;  // processing buffer.  If you need more data to start...
  int32_t      dnext; // Distance to next pointer
};

typedef struct fcomp fcomp;
typedef struct fdecomp fdecomp;

fcomp* fcomp_new();
//    Makes a new fcomp object and returns a pointer (no c++).
fdecomp* fdecomp_new();
//    Makes a new fdecomp object and returns a pointer (no c++).

int32_t fcomp_push(fcomp* hInst, int32_t lob, char* stream);
//    stream to compress and put onto end of fcomp.
char* fcomp_pull(fcomp* hInst, int32_t lob);
//    Pulls compressed data from the beginning of the fcomp and returns it.  If
 //   the buffer is not lob long, it returns 0.
int32_t fcomp_size(fcomp* hInst);
//    Returns hInst->lob.
int32_t fcomp_delete(fcomp* hInst);
//    delete an fcomp.

int32_t fdecomp_push(fdecomp* hInst, int32_t lob, char* stream);
//    stream to decompress and put onto end of fdecomp.
char* fdecomp_pull(fdecomp* hInst, int32_t lob);
//    Pulls uncompressed data from the beginning of the fdecomp and returns it.
//    If the buffer is not lob long, it returns 0.
int32_t fdecomp_size(fdecomp* hInst);
//     Returns hInst->lob.
int32_t fdecomp_delete(fdecomp* hInst);
  //  delete an fdecomp.

/*
As you can see the API is quite simple.  both new()'s malloc new fcomps and
initialize them.  Both delete's free the data properly.  To use this, just push
the input data, ask for the size, and pull the output data.  Pulling is
destructive, i.e. the data is actually removed from the object.

fdecomp's automatically identify fcomp-extended streams.
*/
#endif /*_FCOMP__H_*/

